@@ -11,6 +11,7 @@ import (
 	"github.com/siderolabs/go-pointer"
 	"github.com/siderolabs/talos/pkg/machinery/cel"
 	"github.com/siderolabs/talos/pkg/machinery/cel/celenv"
+	cinterfaces "github.com/siderolabs/talos/pkg/machinery/config/config"
 	"github.com/siderolabs/talos/pkg/machinery/config/types/network"
 	"github.com/siderolabs/talos/pkg/machinery/config/types/v1alpha1"
 	"github.com/siderolabs/talos/pkg/machinery/nethelpers"
@@ -346,7 +347,11 @@ func GenerateBondConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 			continue
 		}
 
-		bondConfig := GenerateBondConfig(device)
+		bondConfig, err := GenerateBondConfig(device)
+		if err != nil {
+			return nil, err
+		}
+
 		if bondConfig == nil {
 			continue
 		}
@@ -366,9 +371,9 @@ func GenerateBondConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 	return CombineYamlBytes(result), nil
 }
 
-func GenerateBondConfig(device *v1alpha1.Device) *network.BondConfigV1Alpha1 {
+func GenerateBondConfig(device *v1alpha1.Device) (*network.BondConfigV1Alpha1, error) {
 	if device == nil || device.DeviceBond == nil {
-		return nil
+		return nil, nil
 	}
 
 	bondConfig := network.NewBondConfigV1Alpha1(device.DeviceInterface)
@@ -485,94 +490,42 @@ func GenerateBondConfig(device *v1alpha1.Device) *network.BondConfigV1Alpha1 {
 		}
 	}
 
-	addCommonLinkConfig(&bondConfig.CommonLinkConfig, device)
+	if err := addCommonLinkConfig(&bondConfig.CommonLinkConfig, device); err != nil {
+		return nil, err
+	}
 
-	return bondConfig
+	return bondConfig, nil
 }
 
-func GenerateDHCP4ConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
+func GenerateDHCPConfigsBytes(devices []*v1alpha1.Device) ([]byte, error) {
 	var result [][]byte
 
 	for _, device := range devices {
-		if device.DeviceDHCP == nil || !*device.DeviceDHCP {
-			continue
+		dhcpConfigs := GenerateDHCPConfigs(device)
+		if len(dhcpConfigs) > 0 {
+			for _, dhcpConfig := range dhcpConfigs {
+				dhcpBytes, err := marshalYaml(dhcpConfig)
+				if err != nil {
+					return nil, err
+				}
+				result = append(result, dhcpBytes)
+			}
 		}
-
-		dhcpConfig := GenerateDHCP4Config(device)
-		if dhcpConfig == nil {
-			continue
-		}
-
-		dhcpBytes, err := marshalYaml(dhcpConfig)
-		if err != nil {
-			return nil, err
-		}
-
-		result = append(result, dhcpBytes)
 	}
 
-	if len(result) == 0 {
-		return nil, nil
+	if len(result) > 0 {
+		return CombineYamlBytes(result), nil
 	}
-	return CombineYamlBytes(result), nil
+
+	return nil, nil
 }
 
-func GenerateDHCP4Config(device *v1alpha1.Device) *network.DHCPv4ConfigV1Alpha1 {
-	if device == nil || device.DeviceDHCP == nil || !*device.DeviceDHCP {
-		return nil
+func GenerateDHCPConfigs(device *v1alpha1.Device) []cinterfaces.NetworkDHCPConfig {
+	if device != nil && device.DHCP() {
+		return generateDHCPConfigs(device.DeviceInterface, device.DeviceDHCPOptions)
 	}
 
-	dhcpConfig := network.NewDHCPv4ConfigV1Alpha1(device.DeviceInterface)
-
-	if device.DeviceDHCPOptions != nil {
-		if device.DeviceDHCPOptions.DHCPRouteMetric != 0 {
-			dhcpConfig.ConfigRouteMetric = device.DeviceDHCPOptions.DHCPRouteMetric
-		}
-	}
-
-	return dhcpConfig
-}
-
-func GenerateDHCP6ConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
-	var result [][]byte
-
-	for _, device := range devices {
-		if device.DeviceDHCPOptions == nil || device.DeviceDHCPOptions.DHCPIPv6 == nil || !*device.DeviceDHCPOptions.DHCPIPv6 {
-			continue
-		}
-
-		dhcp6Config := GenerateDHCP6Config(device)
-		if dhcp6Config == nil {
-			continue
-		}
-
-		dhcp6Bytes, err := marshalYaml(dhcp6Config)
-		if err != nil {
-			return nil, err
-		}
-
-		result = append(result, dhcp6Bytes)
-	}
-
-	if len(result) == 0 {
-		return nil, nil
-	}
-
-	return CombineYamlBytes(result), nil
-}
-
-func GenerateDHCP6Config(device *v1alpha1.Device) *network.DHCPv6ConfigV1Alpha1 {
-	if device == nil || device.DeviceDHCPOptions == nil || device.DeviceDHCPOptions.DHCPIPv6 == nil || !*device.DeviceDHCPOptions.DHCPIPv6 {
-		return nil
-	}
-
-	dhcp6Config := network.NewDHCPv6ConfigV1Alpha1(device.DeviceInterface)
-
-	if device.DeviceDHCPOptions.DHCPRouteMetric != 0 {
-		dhcp6Config.ConfigRouteMetric = device.DeviceDHCPOptions.DHCPRouteMetric
-	}
-
-	return dhcp6Config
+	return nil
 }
 
 func GenerateVIPConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
@@ -622,7 +575,11 @@ func GenerateLinkConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 			continue
 		}
 
-		linkConfig := GenerateLinkConfig(device)
+		linkConfig, err := GenerateLinkConfig(device)
+		if err != nil {
+			return nil, err
+		}
+
 		if linkConfig == nil {
 			continue
 		}
@@ -642,9 +599,9 @@ func GenerateLinkConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 	return CombineYamlBytes(result), nil
 }
 
-func GenerateLinkConfig(device *v1alpha1.Device) *network.LinkConfigV1Alpha1 {
+func GenerateLinkConfig(device *v1alpha1.Device) (*network.LinkConfigV1Alpha1, error) {
 	if device == nil {
-		return nil
+		return nil, nil
 	}
 
 	hasAddresses := len(device.DeviceAddresses) > 0
@@ -652,7 +609,7 @@ func GenerateLinkConfig(device *v1alpha1.Device) *network.LinkConfigV1Alpha1 {
 	hasMTU := device.DeviceMTU > 0
 
 	if !hasAddresses && !hasRoutes && !hasMTU {
-		return nil
+		return nil, nil
 	}
 
 	linkConfig := network.NewLinkConfigV1Alpha1(device.DeviceInterface)
@@ -660,9 +617,9 @@ func GenerateLinkConfig(device *v1alpha1.Device) *network.LinkConfigV1Alpha1 {
 	for _, address := range device.DeviceAddresses {
 		prefix, err := netip.ParsePrefix(address)
 		if err != nil {
-			ip, ipErr := netip.ParseAddr(address)
-			if ipErr != nil {
-				continue
+			ip, err := netip.ParseAddr(address)
+			if err != nil {
+				return nil, err
 			}
 			bits := 32
 			if ip.Is6() {
@@ -679,7 +636,7 @@ func GenerateLinkConfig(device *v1alpha1.Device) *network.LinkConfigV1Alpha1 {
 	for _, route := range device.DeviceRoutes {
 		routeConfig, err := buildRouteConfig(route)
 		if err != nil {
-			continue
+			return nil, err
 		}
 		linkConfig.LinkRoutes = append(linkConfig.LinkRoutes, *routeConfig)
 	}
@@ -688,7 +645,7 @@ func GenerateLinkConfig(device *v1alpha1.Device) *network.LinkConfigV1Alpha1 {
 		linkConfig.LinkMTU = uint32(device.DeviceMTU)
 	}
 
-	return linkConfig
+	return linkConfig, nil
 }
 
 func GenerateVLANConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
@@ -700,17 +657,18 @@ func GenerateVLANConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 		}
 
 		for _, vlan := range device.DeviceVlans {
-			vlanConfig := GenerateVLANConfig(device, vlan)
-			if vlanConfig == nil {
-				continue
-			}
-
-			vlanBytes, err := marshalYaml(vlanConfig)
+			vlanConfigs, err := GenerateVLANConfig(device, vlan)
 			if err != nil {
 				return nil, err
 			}
 
-			result = append(result, vlanBytes)
+			for _, vlanConfig := range vlanConfigs {
+				bytes, err := marshalYaml(vlanConfig)
+				if err != nil {
+					return nil, err
+				}
+				result = append(result, bytes)
+			}
 		}
 	}
 
@@ -721,12 +679,14 @@ func GenerateVLANConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 	return CombineYamlBytes(result), nil
 }
 
-func GenerateVLANConfig(device *v1alpha1.Device, vlan *v1alpha1.Vlan) *network.VLANConfigV1Alpha1 {
+func GenerateVLANConfig(device *v1alpha1.Device, vlan *v1alpha1.Vlan) ([]cinterfaces.NamedDocument, error) {
 	if device == nil || vlan == nil {
-		return nil
+		return nil, nil
 	}
 
 	if vlan.VlanID > 0 {
+		var docs []cinterfaces.NamedDocument
+
 		vlanInterface := fmt.Sprintf("%s.%d", device.DeviceInterface, vlan.VlanID)
 		vlanConfig := network.NewVLANConfigV1Alpha1(vlanInterface)
 		vlanConfig.VLANIDConfig = vlan.VlanID
@@ -766,10 +726,21 @@ func GenerateVLANConfig(device *v1alpha1.Device, vlan *v1alpha1.Vlan) *network.V
 			vlanConfig.LinkMTU = uint32(vlan.VlanMTU)
 		}
 
-		return vlanConfig
+		docs = append(docs, vlanConfig)
+
+		if vlan.DHCP() {
+			dhcpConfigs := generateDHCPConfigs(vlanInterface, vlan.VlanDHCPOptions)
+			if len(dhcpConfigs) > 0 {
+				for _, dhcpConfig := range dhcpConfigs {
+					docs = append(docs, dhcpConfig)
+				}
+			}
+		}
+
+		return docs, nil
 	}
 
-	return nil
+	return nil, nil
 }
 
 func GenerateWireguardConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
@@ -780,7 +751,11 @@ func GenerateWireguardConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 			continue
 		}
 
-		wgConfig := GenerateWireguardConfig(device)
+		wgConfig, err := GenerateWireguardConfig(device)
+		if err != nil {
+			return nil, err
+		}
+
 		if wgConfig == nil {
 			continue
 		}
@@ -800,9 +775,9 @@ func GenerateWireguardConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 	return CombineYamlBytes(result), nil
 }
 
-func GenerateWireguardConfig(device *v1alpha1.Device) *network.WireguardConfigV1Alpha1 {
+func GenerateWireguardConfig(device *v1alpha1.Device) (*network.WireguardConfigV1Alpha1, error) {
 	if device == nil || device.DeviceWireguardConfig == nil {
-		return nil
+		return nil, nil
 	}
 
 	wgConfig := network.NewWireguardConfigV1Alpha1(device.DeviceInterface)
@@ -851,9 +826,11 @@ func GenerateWireguardConfig(device *v1alpha1.Device) *network.WireguardConfigV1
 		}
 	}
 
-	addCommonLinkConfig(&wgConfig.CommonLinkConfig, device)
+	if err := addCommonLinkConfig(&wgConfig.CommonLinkConfig, device); err != nil {
+		return nil, err
+	}
 
-	return wgConfig
+	return wgConfig, nil
 }
 
 func GenerateBridgeConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
@@ -864,7 +841,11 @@ func GenerateBridgeConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 			continue
 		}
 
-		bridgeConfig := GenerateBridgeConfig(device)
+		bridgeConfig, err := GenerateBridgeConfig(device)
+		if err != nil {
+			return nil, err
+		}
+
 		if bridgeConfig == nil {
 			continue
 		}
@@ -884,9 +865,9 @@ func GenerateBridgeConfigBytes(devices []*v1alpha1.Device) ([]byte, error) {
 	return CombineYamlBytes(result), nil
 }
 
-func GenerateBridgeConfig(device *v1alpha1.Device) *network.BridgeConfigV1Alpha1 {
+func GenerateBridgeConfig(device *v1alpha1.Device) (*network.BridgeConfigV1Alpha1, error) {
 	if device == nil || device.DeviceBridge == nil {
-		return nil
+		return nil, nil
 	}
 
 	bridgeConfig := network.NewBridgeConfigV1Alpha1(device.DeviceInterface)
@@ -901,17 +882,44 @@ func GenerateBridgeConfig(device *v1alpha1.Device) *network.BridgeConfigV1Alpha1
 		}
 	}
 
-	addCommonLinkConfig(&bridgeConfig.CommonLinkConfig, device)
+	if err := addCommonLinkConfig(&bridgeConfig.CommonLinkConfig, device); err != nil {
+		return nil, err
+	}
 
-	return bridgeConfig
+	return bridgeConfig, nil
+}
+
+func generateDHCPConfigs(name string, options *v1alpha1.DHCPOptions) []cinterfaces.NetworkDHCPConfig {
+	if options == nil {
+		options = &v1alpha1.DHCPOptions{}
+	}
+
+	result := []cinterfaces.NetworkDHCPConfig{}
+
+	if options.IPv4() {
+		ipv4 := network.NewDHCPv4ConfigV1Alpha1(name)
+		if options.RouteMetric() > 0 {
+			ipv4.ConfigRouteMetric = options.RouteMetric()
+		}
+		result = append(result, ipv4)
+	}
+
+	if options.IPv6() {
+		ipv6 := network.NewDHCPv6ConfigV1Alpha1(name)
+		if options.RouteMetric() > 0 {
+			ipv6.ConfigRouteMetric = options.RouteMetric()
+		}
+		result = append(result, ipv6)
+	}
+
+	return result
 }
 
 func hasSpecialConfig(device *v1alpha1.Device) bool {
 	if device == nil {
 		return false
 	}
-	return device.DeviceBond != nil || len(device.DeviceVlans) > 0 ||
-		device.DeviceWireguardConfig != nil || device.DeviceBridge != nil
+	return device.DeviceBond != nil || device.DeviceWireguardConfig != nil || device.DeviceBridge != nil
 }
 
 func buildRouteConfig(route interface {
@@ -920,38 +928,39 @@ func buildRouteConfig(route interface {
 	Source() string
 	Metric() uint32
 	MTU() uint32
-}) (*network.RouteConfig, error) {
-	networkStr := route.Network()
-	if networkStr == "" {
-		return nil, fmt.Errorf("route network is empty")
-	}
-
-	prefix, err := netip.ParsePrefix(networkStr)
-	if err != nil {
-		return nil, fmt.Errorf("invalid network prefix: %w", err)
-	}
-
+},
+) (*network.RouteConfig, error) {
 	routeConfig := &network.RouteConfig{}
 
-	// For default routes (0.0.0.0/0 or ::/0), omit the destination field
-	// and let Talos infer it from the gateway's address family
-	isDefaultRoute := (prefix.String() == "0.0.0.0/0" || prefix.String() == "::/0")
-	if !isDefaultRoute {
-		routeConfig.RouteDestination = network.Prefix{Prefix: prefix}
+	networkStr := route.Network()
+	if networkStr != "" {
+		prefix, err := netip.ParsePrefix(networkStr)
+		if err != nil {
+			return nil, fmt.Errorf("invalid network prefix: %w", err)
+		}
+
+		// For default routes (0.0.0.0/0 or ::/0), omit the destination field
+		// and let Talos infer it from the gateway's address family
+		isDefaultRoute := (prefix.String() == "0.0.0.0/0" || prefix.String() == "::/0")
+		if !isDefaultRoute {
+			routeConfig.RouteDestination = network.Prefix{Prefix: prefix}
+		}
 	}
 
 	if route.Gateway() != "" {
 		gateway, err := netip.ParseAddr(route.Gateway())
-		if err == nil {
-			routeConfig.RouteGateway = network.Addr{Addr: gateway}
+		if err != nil {
+			return nil, err
 		}
+		routeConfig.RouteGateway = network.Addr{Addr: gateway}
 	}
 
 	if route.Source() != "" {
 		source, err := netip.ParseAddr(route.Source())
-		if err == nil {
-			routeConfig.RouteSource = network.Addr{Addr: source}
+		if err != nil {
+			return nil, err
 		}
+		routeConfig.RouteSource = network.Addr{Addr: source}
 	}
 
 	if route.Metric() > 0 {
@@ -965,17 +974,17 @@ func buildRouteConfig(route interface {
 	return routeConfig, nil
 }
 
-func addCommonLinkConfig(linkConfig *network.CommonLinkConfig, device *v1alpha1.Device) {
+func addCommonLinkConfig(linkConfig *network.CommonLinkConfig, device *v1alpha1.Device) error {
 	if device == nil || linkConfig == nil {
-		return
+		return nil
 	}
 
 	for _, address := range device.DeviceAddresses {
 		prefix, err := netip.ParsePrefix(address)
 		if err != nil {
-			ip, ipErr := netip.ParseAddr(address)
-			if ipErr != nil {
-				continue
+			ip, err := netip.ParseAddr(address)
+			if err != nil {
+				return err
 			}
 			bits := 32
 			if ip.Is6() {
@@ -992,7 +1001,7 @@ func addCommonLinkConfig(linkConfig *network.CommonLinkConfig, device *v1alpha1.
 	for _, route := range device.DeviceRoutes {
 		routeConfig, err := buildRouteConfig(route)
 		if err != nil {
-			continue
+			return err
 		}
 		linkConfig.LinkRoutes = append(linkConfig.LinkRoutes, *routeConfig)
 	}
@@ -1000,6 +1009,8 @@ func addCommonLinkConfig(linkConfig *network.CommonLinkConfig, device *v1alpha1.
 	if device.DeviceMTU > 0 {
 		linkConfig.LinkMTU = uint32(device.DeviceMTU)
 	}
+
+	return nil
 }
 
 // marshalYaml encodes `in` into `yaml` bytes with 2 indentation.
