@@ -3,12 +3,14 @@ package talos
 import (
 	"fmt"
 	"log/slog"
+	"time"
 
 	"github.com/budimanjojo/talhelper/v3/pkg/config"
+	clientconfig "github.com/siderolabs/talos/pkg/machinery/client/config"
 	"github.com/siderolabs/talos/pkg/machinery/config/generate"
 )
 
-func GenerateClientConfigBytes(c *config.TalhelperConfig, input *generate.Input, disableNodesSection bool) ([]byte, error) {
+func GenerateClientConfigBytes(c *config.TalhelperConfig, input *generate.Input, disableNodesSection bool, crtTTL time.Duration) ([]byte, error) {
 	var endpoints []string
 	for _, node := range c.Nodes {
 		if node.ControlPlane {
@@ -16,13 +18,15 @@ func GenerateClientConfigBytes(c *config.TalhelperConfig, input *generate.Input,
 		}
 	}
 	slog.Debug(fmt.Sprintf("endpoints in talosconfig are set to %s", endpoints))
-	input.Options.EndpointList = endpoints
 
-	cfg, err := input.Talosconfig()
+	slog.Debug(fmt.Sprintf("generating admin certificate with TTL of %s", crtTTL))
+	cert, err := input.Options.SecretsBundle.GenerateTalosAPIClientCertificateWithTTL(input.Options.Roles, crtTTL)
 	if err != nil {
 		return nil, err
 	}
 
+	cfg := clientconfig.NewConfig(input.ClusterName, endpoints, input.Options.SecretsBundle.Certs.OS.Crt, cert)
+
 	// The talos production recommendations recommend explicitly setting --node flags and no default nodes.
 	if !disableNodesSection {
 		slog.Debug("appending all nodes to nodes in talosconfig")
